//
//  DBAlgorithmCalculate.h
//  DBAlgorithm
//
//  Created by long yang on 2020/7/8.
//  Copyright © 2020 long yang. All rights reserved.
//

#import <Foundation/Foundation.h>

NS_ASSUME_NONNULL_BEGIN

typedef NS_ENUM(NSInteger,DBControlType) {
    DBControlTypeGetVersion,// 获取设备版本信息
    DBControlTypeGetBattery,// 获取设备电量
    DBControlTypeOpenDataSwitch,// 打开数据通道
    DBControlTypeCloseDataSwitch,// 关闭数据通道
};

typedef NS_ENUM(NSInteger, DBErrorCode){
    DBErrorCodeSuccess,// 请求操作成功
    DBErrorCodePrepareData ,// 数据不完整，继续等待数据
    DBErrorCodeFailure, //内部错误；
    DBErrorCodeNetworkError, // 网络错误;
    DBErrorCodeTimeout, // 内部处理超时，请重试;
    DBErrorCodeParameterInvalid,// 传入参数不合法
    DBErrorCodeMissRawData,// 缺失原始数据
    DBErrorCodeInvalidUser,// 未经授权，无法使用SDK
};

typedef NS_ENUM(NSInteger,DBSignalQuailtyType) {
    DBSignalQuailtyTypeAF7, // AF7通道
    DBSignalQuailtyTypeAF8, // AF8通道
};

/**
 父类model
 */
@interface DBModel : NSObject

-(DBErrorCode)getErrorCode;

@end

/**
 eeg数据model
 */
@interface DBEegModel : DBModel

@property (nonatomic) BOOL detectOK; // 解析成功标志
@property (nonatomic) BOOL haveSecTime; // 此帧数据是否包含80标志位
@property (nonatomic) NSArray * af7Eegs; // AF7通道eeg数据
@property (nonatomic) NSArray * af8Eegs; // AF8通道eeg数据
@property (nonatomic) NSArray * af7Leadoffs; // AF7通道佩戴数据
@property (nonatomic) NSArray * af8Leadoffs; // AF8通道佩戴数据
@property (nonatomic) int counter; // 第几个4秒标志位
@property (nonatomic) int flag;  //时间标志在数据帧中的位置
@property (nonatomic) int index;  // 帧序号

@end

/**
 能量值model
 */
@interface DBAlgorithmEnergyModel : DBModel

@property (nonatomic) double alphaRelative; // alpha相对能量值
@property (nonatomic) double betaRelative; // beta相对能量值
@property (nonatomic) double deltaRelative; // delta相对能量值
@property (nonatomic) double thetaRelative; // theta相对能量值
@property (nonatomic) double gammaRelative; // gamma相对能量值
@property (nonatomic) double alphaAbsolute; // alpha绝对能量值
@property (nonatomic) double betaAbsolute; // beta绝对能量值
@property (nonatomic) double deltaAbsolute; // delta绝对能量值
@property (nonatomic) double thetaAbsolute; // theta绝对能量值
@property (nonatomic) double gammaAbsolute; // gamma绝对能量值

@end

/**
 算法指标model
 */
@interface DBAlgorithmModel : DBModel

@property (nonatomic) double mindfulness; // 正念指标
@property (nonatomic) int focus; // 专注指标
@property (nonatomic) int stress; // 压力指标
@property (nonatomic) int fatigue; // 疲劳指标
@property (nonatomic) int relaxation; // 放松指标
@property (nonatomic) double emotion; // 情绪指标

-(int)getSignalQuailty:(DBSignalQuailtyType)type;// 获取AF7 AF8通道的信号质量(0是信号正常 1是异常)
-(DBAlgorithmEnergyModel *)getEnergy;// 获取能量值

@end

@interface DBRadarModel : DBModel


@property (nonatomic) int stability;  //正念稳定度
@property (nonatomic) float percent;  //正念占比时间
@property (nonatomic) int score;  //正念得分
@property (nonatomic) float potential;  //正念潜力值

@end

/**
 主入口
 */
@interface DBAlgorithmCalculate : NSObject

+(DBAlgorithmCalculate *)sharedInstance;

/**
 初始化授权SDK函数
 */
-(void)startWithToken:(NSString *)token accountId:(NSString *)accountId completion:(void(^)(DBErrorCode))completion;

/**
 增加设备名称（不是地址），在连接好设备之后调用，返回的index用于标识该台设备
 */
-(int)addDevice:(NSString *)deviceName;

/**
 删除设备名称（不是地址），在不用算法之后调用
 */
-(int)deleteDevice:(NSString *)deviceName;

/**
 获取蓝牙设备控制命令，入参是控制命令和设备名称
 */
-(NSData *)getControlDirectBy:(DBControlType)type deviceName:(NSString *)deviceName;

/**
 打开原始数据传输开关，默认关闭
 */
-(void)setRawDataSwitch:(BOOL)isOpenRaw;

/**
 传入蓝牙获取到的字符串，解析成eeg数据，并返回算法指标
 */
-(void)parseRawString:(NSString *)raw deviceIndex:(int)deviceIndex eegCompletion:(void(^)(DBEegModel *))eegCompletion completion:(void(^)(DBAlgorithmModel *))completion;

/**
 根据512个AF7的原始数据+512个AF8的原始数据组成的数组，获取到计算正念的属性值
 */
-(double)getMindfullness:(NSArray *)raw;

/**
 根据获取到的getMindfullness组成的数组，获取正念的四个维度值
 */
-(void)getMindfullnessScore:(NSArray *)array completion:(void(^)(DBRadarModel *))completion;

/**
 实时量化压力值
 */
-(int)mapStress:(double)stress;

/**
 实时量化专注度指标
 */
-(int)mapFocus:(double)focus;

/**
 实时量化疲劳值
 */
-(int)mapFatigue:(double)fatigue;

/**
 判断SDK鉴权
 */
-(BOOL)isCanUseSDK;

/**
 清空SDK
 */
-(void)close;

@end

NS_ASSUME_NONNULL_END
